/**
 * @file tdd_ir_driver.h
 * @author www.tuya.com
 * @brief tdd_ir_driver module is used to 
 * @version 0.1
 * @date 2022-06-09
 *
 * @copyright Copyright (c) tuya.inc 2022
 *
 */

#ifndef __TDD_IR_DRIVER_H__
#define __TDD_IR_DRIVER_H__

#include "tdl_ir_driver.h"

#ifdef __cplusplus
extern "C" {
#endif

/***********************************************************
************************macro define************************
***********************************************************/


/***********************************************************
***********************typedef define***********************
***********************************************************/

typedef unsigned char IR_DRIVER_TYPE_E;
#define IR_DRV_SINGLE_TIMER     0
#define IR_DRV_DUAL_TIMER       1
#define IR_DRV_CAPTURE          2
#define IR_DRV_TYPE_MAX         3

typedef struct {
    int send_pin; /* This pin should support 38kHz pwm output */
    int recv_pin;
    int send_timer;
    int recv_timer; /* Use only in IR_DRV_DUAL_TIMER */
    int send_duty;
}IR_DRV_CFG_T;

/***********************************************************
********************function declaration********************
***********************************************************/

/**
 * @brief register ir driver, maximum 16 bytes
 *
 * @param[in] dev_name: device name, 
 * @param[in] drv_cfg: driver config params
 *
 * @return OPRT_OK on success. Others on error, please refer to "tuya_error_code.h"
 * 
 * @note 
 * 对于 BK7231N 平台：
 *      为了保证红外发送码值的精准度，在红外发送期间会屏蔽 fiq 和 32k 定时器中断。
 *      屏蔽 fiq 中断会导致在红外发送期间无法进行无线数据的通信。
 *      屏蔽 32k 定时器会导致在红外发送期间无法进行任务调度，硬件定时器 2, 3 也会因此变得不准确。
 *      硬件定时器 0， 1 并不会受到影响，产品开发包中红外默认使用了硬件定时器 0。
 * 
 * For the BK7231N platform:
 *      In order to ensure the accuracy of infrared transmission codes, 
 *      the FIQ and 32k timer interrupts are disabled during infrared transmission.
 *      Disabling the FIQ interrupt prevents wireless data communication during infrared transmission.
 *      Disabling the 32k timer interrupt prevents task scheduling during infrared transmission, 
 *      and hardware timers 2 and 3 may become less accurate as a result.
 *      Hardware timers 0 and 1 are not affected, 
 *      and the default infrared usage in the product development kit is with hardware timer 0.
 */
int tdd_ir_driver_register(CHAR_T *dev_name, IR_DRIVER_TYPE_E driver_type, IR_DRV_CFG_T drv_cfg);

#ifdef __cplusplus
}
#endif

#endif /* __TDD_IR_DRIVER_H__ */
